globals 
[
  fast average slow stay-put      ;; current counts
  avg-speed avg-kinetic-energy    ;; current averages
  mean-wall-collisions            ;; mean number of collisions per particle with wall
  mean-particle-collisions        ;; mean number of collisions per particle with other particles
  avg-speed-init avg-energy-init  ;; initial averages
  clock vsplit vclock             ;; clock variables
  box-edge                        ;; patch coords of box's edge
]

turtles-own 
[
  speed  kinetic-energy  mass          ;; turtle info
  wall-collisions
  wall-collisions-history
  particle-collisions
  particle-collisions-history
  v1t v1l tmp-turtle                    ;; collision info (turtle 1)
  heading2 mass2 speed2 v2t v2l turtle2 ;; collision info (turtle 2)
  theta                                 ;; collision info (both turtles)
]

to setup
  locals [ my-patch patches-in-box ]
  if particles-take-up-space? and one-dimension-motion? 
    [ user-message "The model cannot combine one-dimensional motion and particles taking up space. Please change your settings."  stop ]
  ca
  set box-edge (round (screen-edge-x * box-size / 100))
  set patches-in-box ((box-edge - 1) * (box-edge - 1) * 4 + 4 * box-edge + 1 )
  if ( particles-take-up-space? and box?  and number > patches-in-box )
    [ user-message "The box is too small for the number of particles. Please change your settings."  stop ]
  if box? [ make-box ]
  set clock 0
  set vclock 0
  set mean-wall-collisions 0
  set mean-particle-collisions 0
  cct number 
  [ set speed 10
    set mass particle-mass
    set wall-collisions-history [0]
    set particle-collisions-history [0]
    if one-dimension-motion? [ set heading 90 + random-int-or-float 2 * 180 ]
    ifelse ( box? and particles-in-box?) 
    [ ifelse not particles-take-up-space? 
      [ random-position 
      ] 
      [ set my-patch random-one-of patches with 
        [not any? turtles-here and pxcor < box-edge and pxcor > ( - box-edge) and pycor < box-edge and pycor > ( - box-edge )]
        setxy (pxcor-of my-patch) (pycor-of my-patch)
      ]
    ] 
    [ ifelse not particles-take-up-space? 
      [ setxy (random-int-or-float screen-size-x) (random-int-or-float screen-size-y) ] 
      [ set my-patch random-one-of patches with [not any? turtles-here ]
        setxy (pxcor-of my-patch) (pycor-of my-patch)
      ] 
    ] 
    set shape "circle"
    recolor
  ]
  update-variables
  set avg-speed-init avg-speed
  set avg-energy-init avg-kinetic-energy
  setup-plots
  do-plotting
end

to update-variables
  ask turtles
    [ set kinetic-energy (0.5 * speed * speed * mass) ]
  set average count turtles with [color = green]
  set slow    count turtles with [color = blue]
  set fast    count turtles with [color = red]
  set stay-put count turtles with [color = white]
  set avg-speed  mean values-from turtles [speed]
  set avg-kinetic-energy mean values-from turtles [kinetic-energy]
  set vsplit ceiling max values-from turtles [speed]
  set mean-wall-collisions 0.1 * (mean values-from turtles [mean wall-collisions-history])
  set mean-particle-collisions 0.1 * (mean values-from turtles [mean particle-collisions-history])
end

to recalculate-vsplit
  ;; this needs to be done without-interruption to be sure that nothing tries to
  ;; use vsplit or vclock before they've been recalculated
  without-interruption
  [ set vsplit 2 * vsplit
    set vclock 2 * vclock
  ]
end

to go
  if bounce? [ ask turtles [ bounce ]]
  without-interruption 
  [
    ifelse not particles-take-up-space?     
      [ ask turtles [ move ]] 
      [ ask turtles [ move-to-unoccupied-patch ]]
  ]
  set vclock (vclock + 1)
  if (vclock = vsplit)
  [ set clock (clock + 1)
    set vclock 0
    update-variables
    do-plotting
  ]
end

to bounce  ;; turtle procedure
  locals [new-px new-py]
  ; if we're not about to hit a wall (yellow patch),
  ; we don't need to do any further checks
  if pcolor-of patch-at dx dy != yellow 
  [ 
    set wall-collisions 0
    ifelse length wall-collisions-history < 10
      [ set wall-collisions-history fput wall-collisions wall-collisions-history ]
      [ set wall-collisions-history fput wall-collisions (but-last wall-collisions-history) ]
    stop 
  ]
  ; get the coordinates of the patch we'll be on if we go forward 1
  set new-px round (xcor + dx)
  set new-py round (ycor + dy)
  set wall-collisions 1
  ifelse length wall-collisions-history < 10
    [ set wall-collisions-history fput wall-collisions wall-collisions-history ]
    [ set wall-collisions-history fput wall-collisions (but-last wall-collisions-history) ]
  ; check: hitting left or right wall?
  if (abs new-px = box-edge)
    ; if so, reflect heading around x axis
    [ set heading (- heading) ]
  ; check: hitting top or bottom wall?
  if (abs new-py = box-edge)
    ; if so, reflect heading around y axis
    [ set heading (180 - heading) ]
end

to move-to-unoccupied-patch ;; two particles cannot occupy the same space
  locals [ empty-patch new-x new-y ]
  without-interruption
  [
    while [(speed / vsplit) >= 1.0]
      [ recalculate-vsplit ]   
    set new-x ((speed / vsplit) * dx)
    set new-y ((speed / vsplit) * dy)
    if (pcolor-of (patch-at new-x new-y) != black) 
    [
      bounce
      stop
    ]
    ifelse count (turtles-at new-x new-y) with [ who != who-of myself ] = 0
    [
      jump (speed / vsplit)
    ]
    [
      ifelse collisions?
      [ set tmp-turtle random-one-of (turtles-at new-x new-y) with [who != who-of myself]
    ;    if (who > who-of tmp-turtle)
    ;      [ collide ]
          collide
      ]
      [  set empty-patch random-one-of neighbors with [ not any? turtles-here and pcolor = black ] in-radius (speed / vsplit)
         if empty-patch != nobody
          [ 
            set heading towards empty-patch
            jump (speed / vsplit)
          ]          
      ]
     ]
   ]
end


to move  ;; turtle procedure
  locals [ new-x new-y ]
  while [(speed / vsplit) >= 1.0]
  [ recalculate-vsplit ]
  set new-x ((speed / vsplit) * dx)
  set new-y ((speed / vsplit) * dy)
  if (pcolor-of (patch-at new-x new-y) != black) 
  [
    bounce
    stop
  ]
  jump (speed / vsplit)
  if collisions? 
  [ ifelse not one-dimension-motion? 
    [ check-for-collision ] 
    [ check-for-collision-one-dimension ]
  ]
end

to check-for-collision  ;; turtle procedure
  ifelse count other-turtles-here >= 1
    [ set tmp-turtle random-one-of other-turtles-here
      ;if ((who > who-of tmp-turtle) and (turtle2 != tmp-turtle))
      ;  [ 
          collide 
          set particle-collisions 1
          ifelse length particle-collisions-history < 10
            [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
            [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
      ;  ]
    ]
    [
      set particle-collisions 0
      ifelse length particle-collisions-history < 10
        [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
        [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
    ]
end
        
to check-for-collision-one-dimension  ;; turtle procedure
  ifelse count other-turtles-here with [ heading != heading-of myself ] >= 1
    [ set tmp-turtle random-one-of other-turtles-here with [ heading != heading-of myself ]
      if ((who > who-of tmp-turtle) and (turtle2 != tmp-turtle))
        [ 
          collide
          set particle-collisions 1
          ifelse length particle-collisions-history < 10
            [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
            [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
        ]
    ]
    [
      set particle-collisions 0
      ifelse length particle-collisions-history < 10
        [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
        [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
    ]
end

to collide  ;; turtle procedure
  get-turtle2-info
  if not one-dimension-motion? [ calculate-velocity-components ]
  set-new-speed-and-headings
end

to get-turtle2-info  ;; turtle procedure
  set turtle2 tmp-turtle
  set mass2 mass-of turtle2
  set speed2 speed-of turtle2
  set heading2 heading-of turtle2
end

to calculate-velocity-components
  locals [vcm]  ;; CM vel. along dir. theta
  set theta (random-int-or-float 360)
  set v1l (speed * sin (theta - heading))
  set v1t (speed * cos (theta - heading))
  set v2l (speed2 * sin (theta - heading2))
  set v2t (speed2 * cos (theta - heading2))
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2))
  set v1t (vcm + vcm - v1t)
  set v2t (vcm + vcm - v2t)
end

to set-new-speed-and-headings  ;; turtle procedure
  locals [ temp-speed ]
  set temp-speed speed
  ifelse one-dimension-motion? 
  [ set speed speed2 ]
  [ set speed sqrt ((v1t * v1t) + (v1l * v1l)) ]
  ifelse one-dimension-motion? 
      [ set heading heading + 180 ]
      [ set heading (theta - (atan v1l v1t)) ]
  ifelse one-dimension-motion?
      [ set speed-of turtle2 temp-speed ]
      [ set speed-of turtle2 sqrt ((v2t * v2t) + (v2l * v2l)) ]
  ifelse one-dimension-motion?
      [ set heading-of turtle2 heading-of turtle2 + 180 ]
      [ set heading-of turtle2 (theta - (atan v2l v2t)) ]
  recolor
  ask turtle2 
    [ 
      recolor
      set particle-collisions 1
      ifelse length particle-collisions-history < 10
        [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
        [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
    ]
end

to recolor  ;; turtle procedure
  ifelse ( speed < (0.5 * 10) and speed > 0 )
    [ set color blue ]
    [ ifelse speed > (1.5 * 10)
        [ set color red ]
        [ set color green ] ]
end

to make-box
  ask patches with [((abs pxcor = box-edge) and (abs pycor <= box-edge)) or
                    ((abs pycor = box-edge) and (abs pxcor <= box-edge))]
    [ set pcolor yellow ]
end

to random-position ;; turtle procedure
  setxy ((1 - box-edge) + random-int-or-float (2 * box-edge - 1))
        ((1 - box-edge) + random-int-or-float (2 * box-edge - 1))
end

;;; plotting procedures
to setup-plots
  set-current-plot "Speed Counts"
  set-plot-y-range 0 number
end

to do-plotting
  set-current-plot "Speed Counts"
  set-current-plot-pen "fast"
  plot fast
  set-current-plot-pen "medium"
  plot average
  set-current-plot-pen "slow"
  plot slow

  set-current-plot "collisions"
  set-current-plot-pen "wall collisions"
  plot mean-wall-collisions  
  set-current-plot-pen "particle collisions"
  plot mean-particle-collisions
  
  set-current-plot "Average speed"
  plot avg-speed
end



; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo GasLab Gas in a Box model.
; http://ccl.northwestern.edu/netlogo/models/GasLabGasinaBox.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/GasLabGasinaBox
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
280
12
645
398
35
35
5.0
1
10
1
1
1

CC-WINDOW
313
422
561
605
Command Center

MONITOR
296
661
377
710
clock
clock
0
1

MONITOR
313
609
394
658
fast
fast
0
1

MONITOR
502
610
583
659
slow
slow
0
1

SLIDER
10
139
223
172
number
number
1
500
200
1
1
NIL

BUTTON
110
17
185
50
go
go
T
1
T
OBSERVER
T

BUTTON
26
17
101
50
NIL
setup
NIL
1
T
OBSERVER
T

TEXTBOX
18
75
161
104
Objects: Particles and...

TEXTBOX
9
324
132
343
Interactions

MONITOR
406
609
487
658
medium
average
3
1

PLOT
18
581
277
748
Speed counts
Time
Count
0.0
100.0
0.0
100.0
true
false
PENS
"medium" 1.0 0 -11352576 true
"slow" 1.0 0 -16776961 true
"fast" 1.0 0 -65536 true

MONITOR
395
660
497
709
average speed
avg-speed
3
1

PLOT
17
419
278
582
Average speed
Time
Speed
0.0
100.0
0.0
12.0
true
false
PENS
"speed" 1.0 0 -7566196 true

SWITCH
10
249
222
282
particles-take-up-space?
particles-take-up-space?
1
1
-1000

MONITOR
507
661
649
710
average kinetic energy
avg-kinetic-energy
3
1

SLIDER
9
286
223
319
box-size
box-size
0
100
60
1
1
%

SWITCH
9
380
161
413
particles-in-box?
particles-in-box?
0
1
-1000

SWITCH
171
69
275
102
box?
box?
0
1
-1000

SWITCH
165
381
275
414
collisions?
collisions?
0
1
-1000

TEXTBOX
15
115
134
134
Properties

SLIDER
10
175
223
208
particle-mass
particle-mass
2
50
2
1
1
a.m.u.

SWITCH
9
344
113
377
bounce?
bounce?
0
1
-1000

SWITCH
10
214
223
247
one-dimension-motion?
one-dimension-motion?
1
1
-1000

PLOT
662
300
931
526
collisions
time
collisions
0.0
100.0
0.0
0.01
true
true
PENS
"wall collisions" 1.0 0 -44544 true
"particle collisions" 1.0 0 -65413 true

MONITOR
664
230
757
279
wall collisions
mean-wall-collisions
5
1

MONITOR
772
231
884
280
particle collisions
mean-particle-collisions
5
1

TEXTBOX
688
155
878
213
Number of collisions per particle per time tick

@#$#@#$#@
WHAT IS IT?
-----------
This program simulates the behavior of gas molecules.

The Gas-in-a-Box model is one in a collection of GasLab models that use the same basic rules for expressing what happens when gas molecules collide.  Each one has different features in order to show different aspects of the behavior of gases.


HOW IT WORKS
------------
Molecules have kinetic energy (that which is due to their motion).  Collisions between molecules are perfectly elastic.  Molecules are colored according to speed -- blue for slow, green for medium, and red for high speeds.

Coloring of the molecules is based on INITSPEED, molecules with a speed less than 50% of INITSPEED are blue, ones that are more than 50% faster are red, while all in between are green.

The exact way two molecules collide is as follows:
1. Two molecules "collide" if they find themselves on the same patch.
2. A random axis is chosen, as if they are two balls that hit each other and this axis is the line connecting their centers.
3. They exchange momentum and energy along that axis, according to the conservation of momentum and energy.  This calculation is done in the center of mass system.
4. Each turtle is assigned its new velocity, energy, and heading.
5. If a turtle finds itself on or very close to a wall of the container, it "bounces" -- that is, reflects its direction and keeps its same speed.


HOW TO USE IT
-------------
Initial settings:
- BOX-SIZE-PERCENT: size of the box. (percentage of the screen size)
- NUMBER: number of gas molecules
- INITSPEED: initial speed of the molecules
- INITMASS: mass of the molecules

The SETUP button will set the initial conditions.  The GO button will run the simulation.

Other settings:
- TRACE?: Traces the path of one of the molecules.  This path fades over time to make the screen less cluttered.

Monitors:
- FAST, AVERAGE, SLOW: numbers of molecules with different speeds: fast (red), average (green), and slow (blue).
- AVG-SPEED: average speed of the molecules.
- AVG-ENERGY: average kinetic energy of the molecules.
- CLOCK: number of ticks that have run.

Plots:
- SPEED COUNTS: plots the number of molecules in each range of speed.
- SPEED HISTOGRAM: speed distribution of all the molecules.  The gray line is the average value, and the black line is the initial average.
- ENERGY HISTOGRAM: distribution of energies of all the molecules, calculated as  m*v*v/2.  The gray line is the average value, and the black line is the initial average.

Initially, all the molecules have the same speed but random directions. Therefore the first histogram plots of speed and energy should show only one column each.  As the molecules repeatedly collide, they exchange energy and head off in new directions, and the speeds are dispersed -- some molecules get faster, some get slower.


THINGS TO NOTICE
----------------
What is happening to the numbers of molecules of different colors?  Does this match what's happening in the histograms?  Why are there more blue molecules than red ones?

Can you observe collisions and color changes as they happen?  For instance, when a red molecule hits a green molecule, what color do they each become?

Why does the average speed (avg-speed) drop?  Does this violate conservation of energy?

The molecule histograms quickly converge on the classic Maxwell-Boltzmann distribution.  What's special about these curves?  Why is the shape of the energy curve not the same as the speed curve?

Watch the molecule whose path is traced in gray.  Does the trace resemble Brownian motion?  Can you recognize when a collision happens?  What factors affect the frequency of collisions?   What about the "angularity" of the path?  Can you get it to stay "local" or travel all over the screen?

In what ways is this model an incorrect idealization of the real world?


THINGS TO TRY
-------------
Set all the molecules in part of the screen, or with the same heading -- what happens?  Does this correspond to a physical possibility?

Try different settings, especially the extremes.  Are the histograms different?  Does the trace pattern change?

Are there other interesting quantities to keep track of?

Look up or calculate the REAL number, size, mass and speed of molecules in a typical gas.  When you compare those numbers to the ones in the model, are you surprised this model works as well as it does?  What physical phenomena might be observed if there really were a small number of big molecules in the space around us?

We often say outer space is a vacuum.  Is that really true?  How many molecules would there be in a space the size of this computer?


EXTENDING THE MODEL
-------------------
Could you find a way to measure or express the "temperature" of this imaginary gas?  Try to construct a thermometer.

What happens if there are molecules of different masses?  (See GasLab Two Gas model.)

What happens if the collisions are non-elastic?

How does this 2-D model differ from a 3-D model?

Set up only two molecules to collide head-on.  This may help to show how the collision rule works.  Remember that the axis of collision is being randomly chosen each time.

What if some of the molecules had a "drift" tendency -- a force pulling them in one direction?  Could you develop a model of a centrifuge, or charged molecules in an electric field?

Find a way to monitor how often molecules collide, and how far they go between collisions, on the average.  The latter is called the "mean free path".  What factors affect its value?

In what ways is this idealization different from the idealization that is used to derive the Maxwell-Boltzmann distribution?  Specifically, what other code could be used to represent the two-body collisions of molecules?

If MORE than two molecules arrive on the same patch, the current code says they don't collide.  Is this a mistake?  How does it affect the results?

Is this model valid for fluids in any aspect?  How could it be made to be fluid-like?


NETLOGO FEATURES
-----------------
Notice the use of the histogram primitive.

Notice how collisions are detected by the molecules and how the code guarantees that the same two molecules do not collide twice.  What happens if we let the patches detect them?


CREDITS AND REFERENCES
----------------------
This was one of the original Connection Machine StarLogo applications (under the name GPCEE) and is now ported to NetLogo as part of the Participatory Simulations project.

To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo GasLab Gas in a Box model. http://ccl.northwestern.edu/netlogo/models/GasLabGasinaBox. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 1998 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/GasLabGasinaBox for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7566196 true true 35 35 230

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
